//+------------------------------------------------------------------------
//
//  Copyright @ Baris TAZE
//
//  File:     GASettings.cpp
//
//  Content:  Implementation for CGASettings class
//  
//  Classes:  CGASettings
//            
//  History:  27-May-2002    created by baris taze .  
//
//-------------------------------------------------------------------------

#include "StdAfx.h"
#include "GASettings.h"

CGASettings::CGASettings()
{
	m_fCrossOverRate	= GA_SET_DEF_CROSSOVERRATE;
	m_fMutationRate		= GA_SET_DEF_MUTATIONRATE;
	m_nPopSize			= GA_SET_DEF_POPSIZE;

	m_nCrossOverType	= GA_CROSSOVER_TWOPOINT;
	m_fCrsOvrUnfChance	= GA_CRSOVR_DEF_UNIFORM_CHANCE;

	m_nNatSelType		= GA_NATSEL_GENITOR;
	m_nNatSelGntr		= GA_NATSEL_DEF_GENITOR; 
	m_fNatSelTrnmt		= GA_NATSEL_DEF_TOURNAMENT;

	m_nAStopCriteriaType		= AUTO_STOP_CRITERIA_MANUEL;
	m_fAStopCriteriaBias		= DEF_AUTO_STOPBY_BIAS;
	m_nAStopCriteriaMinute		= DEF_AUTO_STOPBY_MINUTE;
	m_nAStopCriteriaIteration	= DEF_AUTO_STOPBY_ITERATION;
	m_nAStopCriteriaRefinement	= DEF_AUTO_STOPBY_REFINEMENT;
}

CGASettings::~CGASettings()
{
}

bool CGASettings::operator == ( CGASettings& arSettings )
{
	if( arSettings.GetCrossOverRate() != m_fCrossOverRate )
		return false;

	if( arSettings.GetMutationRate() != m_fMutationRate )
		return false;

	if( arSettings.GetPopSize() != m_nPopSize )
		return false;

	if( arSettings.GetCrossOverType() != m_nCrossOverType )
		return false;

	if( GA_CROSSOVER_UNIFORM == m_nCrossOverType )
	{
		if( arSettings.GetCrsOvrUnfChance() != m_fCrsOvrUnfChance )
			return false;
	}

	if( arSettings.GetNatSelType() != m_nNatSelType )
		return false;

	if( GA_NATSEL_GENITOR == m_nNatSelType )
	{
		if( arSettings.GetNatSelGntr() != m_nNatSelGntr )
			return false;
	}

	if( GA_NATSEL_TOURNAMENT == m_nNatSelType )
	{
		if( arSettings.GetNatSelTrnmt() != m_fNatSelTrnmt )
			return false;
	}
			
	if( arSettings.GetAStopCriteriaType() != m_nAStopCriteriaType )
		return false;
	
	if( AUTO_STOP_CRITERIA_BIAS == ( AUTO_STOP_CRITERIA_BIAS & m_nAStopCriteriaType ) )
	{
		if( arSettings.GetAStopCriteriaBias() != m_fAStopCriteriaBias )
			return false;
	}

	if( AUTO_STOP_CRITERIA_MINUTE == ( AUTO_STOP_CRITERIA_MINUTE & m_nAStopCriteriaType ) )
	{
		if( arSettings.GetAStopCriteriaMinute() != m_nAStopCriteriaMinute )
			return false;
	}

	if( AUTO_STOP_CRITERIA_ITERATION == ( AUTO_STOP_CRITERIA_ITERATION & m_nAStopCriteriaType ) )
	{
		if( arSettings.GetAStopCriteriaIteration() != m_nAStopCriteriaIteration )
			return false;
	}

	if( AUTO_STOP_CRITERIA_REFINEMENT == ( AUTO_STOP_CRITERIA_REFINEMENT & m_nAStopCriteriaType ) )
	{
		if( arSettings.GetAStopCriteriaRefinement() != m_nAStopCriteriaRefinement )
			return false;
	}	

	return true;
}


double CGASettings::GetCrossOverRate()
{
	return m_fCrossOverRate;
}

void CGASettings::SetCrossOverRate( double afCrossOverRate)
{
	m_fCrossOverRate = afCrossOverRate;
}

double CGASettings::GetMutationRate()
{
	return m_fMutationRate;
}

void CGASettings::SetMutationRate( double afMutationRate )
{
	m_fMutationRate = afMutationRate;
}

long CGASettings::GetPopSize()
{
	return m_nPopSize;
}

void CGASettings::SetPopSize( long anPopSize )
{
	m_nPopSize = anPopSize;
}

long CGASettings::GetCrossOverType()
{
	return m_nCrossOverType;
}

void CGASettings::SetCrossOverType( long anCrossOverType )
{
	m_nCrossOverType = anCrossOverType;
}

double CGASettings::GetCrsOvrUnfChance()
{
	return m_fCrsOvrUnfChance;
}

void CGASettings::SetCrsOvrUnfChance( double afCrsOvrUnfChance )
{
	m_fCrsOvrUnfChance = afCrsOvrUnfChance;
}

long CGASettings::GetNatSelType()
{
	return m_nNatSelType;
}

void CGASettings::SetNatSelType( long anNatSelType )
{
	m_nNatSelType = anNatSelType;
}

long CGASettings::GetNatSelGntr()
{
	return m_nNatSelGntr;
}

void CGASettings::SetNatSelGntr( long anNatSelGntr )
{
	m_nNatSelGntr = anNatSelGntr;
}

double CGASettings::GetNatSelTrnmt()
{
	return m_fNatSelTrnmt;
}

void CGASettings::SetNatSelTrnmt( double afNatSelTrnmt )
{
	m_fNatSelTrnmt = afNatSelTrnmt;
}

//---------------------------------------------//

long CGASettings::GetAStopCriteriaType()
{
	return m_nAStopCriteriaType;
}

void CGASettings::SetAStopCriteriaType( long anAStopCriteriaType )
{
	m_nAStopCriteriaType = anAStopCriteriaType;
}

double CGASettings::GetAStopCriteriaBias()
{
	return m_fAStopCriteriaBias;
}

void CGASettings::SetAStopCriteriaBias( double afAStopCriteriaBias )
{
	m_fAStopCriteriaBias = afAStopCriteriaBias;
}

long CGASettings::GetAStopCriteriaMinute()
{
	return m_nAStopCriteriaMinute;
}

void CGASettings::SetAStopCriteriaMinute( long anAStopCriteriaMinute )
{
	m_nAStopCriteriaMinute = anAStopCriteriaMinute;
}

long CGASettings::GetAStopCriteriaIteration()
{
	return m_nAStopCriteriaIteration;
}

void CGASettings::SetAStopCriteriaIteration( long anAStopCriteriaIteration )
{
	m_nAStopCriteriaIteration = anAStopCriteriaIteration;
}

long CGASettings::GetAStopCriteriaRefinement()
{
	return m_nAStopCriteriaRefinement;
}

void CGASettings::SetAStopCriteriaRefinement( long anAStopCriteriaRefinement )
{
	m_nAStopCriteriaRefinement = anAStopCriteriaRefinement;
}

bool CGASettings::IsAnyAutoStopAssigned()
{
	if( AUTO_STOP_CRITERIA_BIAS == ( AUTO_STOP_CRITERIA_BIAS & m_nAStopCriteriaType ) )
	{
		return true;
	}

	if( AUTO_STOP_CRITERIA_MINUTE == ( AUTO_STOP_CRITERIA_MINUTE & m_nAStopCriteriaType ) )
	{
		return true;
	}
	
	if( AUTO_STOP_CRITERIA_ITERATION == ( AUTO_STOP_CRITERIA_ITERATION & m_nAStopCriteriaType ) )
	{		
		return true;
	}

	if( AUTO_STOP_CRITERIA_REFINEMENT == ( AUTO_STOP_CRITERIA_REFINEMENT & m_nAStopCriteriaType ) )
	{		
		return true;
	}

	return false;
}

void CGASettings::SetAllAutosByDefaultsValues()
{	
	m_fAStopCriteriaBias		= DEF_AUTO_STOPBY_BIAS;
	m_nAStopCriteriaMinute		= DEF_AUTO_STOPBY_MINUTE;
	m_nAStopCriteriaIteration	= DEF_AUTO_STOPBY_ITERATION;
	m_nAStopCriteriaRefinement	= DEF_AUTO_STOPBY_REFINEMENT;
		
	m_nAStopCriteriaType  = AUTO_STOP_CRITERIA_MANUEL;
	m_nAStopCriteriaType |= AUTO_STOP_CRITERIA_BIAS;
	m_nAStopCriteriaType |= AUTO_STOP_CRITERIA_MINUTE;
	m_nAStopCriteriaType |= AUTO_STOP_CRITERIA_ITERATION;
	m_nAStopCriteriaType |= AUTO_STOP_CRITERIA_REFINEMENT;
}
