// GenAlgLib.cpp : Defines the entry point for the console application.
//


/*	
	-This is a very stupid example but we kept it simple to don't make you dig into the driver-
	Toy Problem: Let's say we want to calculate the square root (x) of A = 1,522,756.				 
	Assume that: We have not a mathematical formula to solve this "square root" problem.
	Assume that: Checking all (x*x) values to compare with A to see whether they are equal or not
				 is very expensive; let's say impossible e.g. requires ~ infinite execution time.				 
	Assume that: You will be still happy if we find a solution y which is very near to x but
				 not necessarily exactly equal to x.
	Simple Hint: Exact solution for X = 1234
				 A satisfactory solution y = 1238	whose square is almost equal to A.

    -see CMyGenerator class below for the following comments-

	I, as a designer, decide that: -or- 
	I, as a problem solver, magically know that:
			the solution (x) will be at most a 4-digit number; i.e.
			0<=x<=MAX where MAX = 9999
			To represent 4-digit number I need 14-bits because
			2^14 = 16,384 and it is a little bigger than 9,999
			In your real problem you may calculate this bit-size dynamically.
*/

#include "stdafx.h"
#include <string>

#include "TestDriver.h"
#include "GenAlgCore\Individual.h"
#include "GenAlgCore\GeneratorEx.h"

#define	A	1522756		// my problem is highly static :)
#define	MAX	9999		// max number as a solution for X

// Firstly read the problem definiton above.
// Then review the CMyGenerator below; after that
// you should see this class
//
// This class represent a candidate solution for x.
// The solution is simply a number (x) which has at most 9 digit.
// Therefore this class consists of some bits which means a number.
// We will implement the performance function in this class
class CMyIndividual : public CIndividual<CMyIndividual, CBit>
{
public:
	CMyIndividual(){}

	// funny function to stress what it is
	int GetTheNumberThatThisObjectRepresents()
	{
		// use utilities in base classes
		return this->GetDecimal( 0, this->GetBitSize() );
	}

	// the performance function will be like that:
	// 1 - { absolute_value_of_[A-(x*x)] / (MAX*MAX) }
	// to get a performance value between 0 and 1.
	double CMyIndividual::EvaluatePerformance( DWORD_PTR adwAppData )
	{	
		// In main() function I am passing NULL as application data;
		// Therefore I don't use these adwAppData input here... But
		// you most likely will pass a real object and use it here.
		// It may be a database of macroeconomical datas or stock prices etc.
		// CMyDatabase* pDatabase = (CMyDatabase*)adwAppData;
		// pDatabase->GetStockPriceAt(stockID, dateTime);
		// ...

		// let's return our toy example

		// convert bits into number
		int candidateX = this->GetTheNumberThatThisObjectRepresents();
		
		if(candidateX > MAX)
		{
			return 0.0f;
		}

		long square = (long)candidateX * (long)candidateX;
		long difference = A - square;
		if(difference < 0)
		{
			difference *= -1;	// absolute value
		}
		
		double performance = (double)difference / (double)(MAX*MAX);
		
		performance = 1.0f - performance;
		
		ASSERT(performance >= 0.0f);
		ASSERT(performance <= 1.0f);

		return performance;
	}

};


// override base class to implement init
class CMyGenerator : public CGeneratorEx<CMyIndividual>
{
public:
	void Init()
	{	
		// I, as a designer, decide that: -or- 
		// I, as a problem solver, magically know that:
		// 		the solution (x) will be at most a 4-digit number; i.e.
		// 		0<=x<=MAX where MAX = 9999
		// 		To represent 4-digit number I need 14-bits because
		// 		2^14 = 16,384 and it is a little bigger than 9,999
		// 		In your real problem you may calculate this bit-size dynamically.		int bitSize = 14;
		int bitSize = 14;

		// a meaningfull number... 500, 1000, 3000, 5000 etc. sounds good
		int populationSize = 1000;
		// alter the default settings...
		m_Settings.SetPopSize(populationSize);
		
		// define auto stop criteria... 
		// it is manual by default...
		long autoStopCriteria = 0;
		autoStopCriteria |= AUTO_STOP_CRITERIA_BIAS;
		autoStopCriteria |= AUTO_STOP_CRITERIA_MINUTE;
		autoStopCriteria |= AUTO_STOP_CRITERIA_ITERATION;
		autoStopCriteria |= AUTO_STOP_CRITERIA_REFINEMENT;
		// set auto stop criteria...
		m_Settings.SetAStopCriteriaType(autoStopCriteria);

		// create the world :)
		CreatePopulation( populationSize, bitSize);

		// homework: think about locking some bits to narrow the genetic search :)
		// i.e. let's say that you somehow know that the 18th bit of the real solution
		// is '1'... Then you may enforce the GenAlg library to fix this bit as 1 in
		// whole population and protect it from mutation along the next generations
	}
};


// simple test
void main(int argc, TCHAR* argv[], TCHAR* envp[])
{
	std::cout << "\n\n\tGenetic Algorithms Library\n";
	std::cout << "\tDeveloped by Baris Taze\n\n";	  
	std::cout << "Problem: What is the square root of 1,522,756\n";


	// create generator
	CMyGenerator evolutioner;
	// I don't want to pass an application data to the GeneticAlgorithm black-box.
	// But you may want to pass a pointer of the datase to make some simulation
	// and help your individual to calculate a good performance function...
	evolutioner.SetAppData(NULL);

	// create the world :)
	evolutioner.Init();

	// evolution!!!
	while(!evolutioner.IsConverged())
	{
		// the total execution (while loop) may take 10 minutes.
		// see default values of GASettings member.
		// see IsConverged() function
		evolutioner.ProduceNextGeneration();	
	}
	
	// auto-stop criteria has been reached
	int x = evolutioner.GetHistoricalBest()->GetTheNumberThatThisObjectRepresents();
	
	// apocalypse :)
	evolutioner.Free();

	// here is our solution		
	std::cout << "Actual mathematical solution of the problem is: 1234\n";
	std::cout << "In this run, Genetic Algorithm has found it as: " << x << "\n\n\n";
	
	std::cout << "Enter something to exit" << std::endl;
	std::string str;
	std::cin >> str;
}
